@@ -1,4 +1,3 @@
-#!/usr/bin/python2.4
 #
 # Copyright 2008 Google Inc.
 #
@@ -23,15 +22,15 @@ encrypt, decrypt, sign and verify.
 
 import os
 import warnings
-
-import errors
 import json
-import keydata
-import keyinfo
-import keys
-import readers
-import writers
-import util
+
+from keyczar import errors
+from keyczar import keydata
+from keyczar import keyinfo
+from keyczar import keys
+from keyczar import readers
+from keyczar import writers
+from keyczar import util
 
 VERSION = 0
 VERSION_BYTE = '\x00'
@@ -292,7 +291,7 @@ class GenericKeyczar(Keyczar):
     if isinstance(writer, basestring):
       writer = writers.CreateWriter(writer)
       warnings.warn(
-        'Using a string as the writer is deprecated. Use writers.CreateWriter', 
+        'Using a string as the writer is deprecated. Use writers.CreateWriter',
         DeprecationWarning)
     self.metadata.encrypted = (encrypter is not None)
     writer.WriteMetadata(self.metadata)
@@ -353,7 +352,7 @@ class Encrypter(Keyczar):
     @type output_stream: 'file-like' object
 
     @param encoder: the encoding stream to use on the ciphertext stream.
-    Defaults to base64 encoding with no padding or line breaks. 
+    Defaults to base64 encoding with no padding or line breaks.
     Use None for raw bytes.
     @type encoder: 'file-like' object
 
@@ -390,7 +389,7 @@ class Verifier(Keyczar):
     """Only valid if purpose includes verifying."""
     return purpose == keyinfo.VERIFY or purpose == keyinfo.SIGN_AND_VERIFY
 
-  def Verify(self, data, sig):
+  def Verify(self, data, sig, decoder=util.Base64WSDecode):
     """
     Verifies whether the signature corresponds to the given data.
 
@@ -403,12 +402,12 @@ class Verifier(Keyczar):
     @return: True if sig corresponds to data, False otherwise.
     @rtype: boolean
     """
-    sig_bytes = util.Base64WSDecode(sig)
+    sig_bytes = decoder(sig) if decoder else sig
     if len(sig_bytes) < HEADER_SIZE:
       raise errors.ShortSignatureError(len(sig_bytes))
     return self.__InternalVerify(sig_bytes[:HEADER_SIZE], sig_bytes[HEADER_SIZE:], data)
 
-  def AttachedVerify(self, signed_data, nonce):
+  def AttachedVerify(self, signed_data, nonce, decoder=util.Base64WSDecode):
     """
     Verifies the signature in the signed blob corresponds to the data
     in the signed blob and the provided nonce, and returns the data.
@@ -426,7 +425,7 @@ class Verifier(Keyczar):
     otherwise, None
     @rtype: string
     """
-    decoded_data = util.Base64WSDecode(signed_data)
+    decoded_data = decoder(signed_data) if decoder else signed_data
 
     data, offset = util.UnpackByteArray(decoded_data, HEADER_SIZE)
     signature = decoded_data[offset:]
@@ -462,7 +461,7 @@ class UnversionedVerifier(Keyczar):
     """Only valid if purpose includes verifying."""
     return purpose == keyinfo.VERIFY or purpose == keyinfo.SIGN_AND_VERIFY
 
-  def Verify(self, data, sig):
+  def Verify(self, data, sig, decoder=util.Base64WSDecode):
     """
     Verifies whether the signature corresponds to the given data. This is a
     stanard signature (i.e. HMAC-SHA1, RSA-SHA1, DSA-SHA1) that contains no
@@ -477,7 +476,7 @@ class UnversionedVerifier(Keyczar):
     @return: True if sig corresponds to data, False otherwise.
     @rtype: boolean
     """
-    sig_bytes = util.Base64WSDecode(sig)
+    sig_bytes = decoder(sig) if decoder else sig
 
     for version in self.versions:
       key = self._keys[version]
@@ -548,7 +547,7 @@ class Crypter(Encrypter):
     @type output_stream: 'file-like' object
 
     @param decoder: the decoding stream to use on the incoming stream.
-    Defaults to base64 decoding with no padding or line breaks. 
+    Defaults to base64 decoding with no padding or line breaks.
     Use None for handling raw bytes.
     @type decoder: 'file-like' object
 
@@ -587,11 +586,11 @@ class Signer(Verifier):
     """Only valid if purpose includes signing."""
     return purpose == keyinfo.SIGN_AND_VERIFY
 
-  def Sign(self, data):
+  def Sign(self, data, encoder=util.Base64WSEncode):
     """
     Sign given data and return corresponding signature.
 
-    For message M, outputs the signature as Header|Sig(Header.M).
+    For message M, outputs the signature as Header|Sig(M|VersionByte).
 
     @param data: message to be signed
     @type data: string
@@ -599,15 +598,15 @@ class Signer(Verifier):
     @return: signature on the data encoded as a Base64 string
     @rtype: string
     """
-    return util.Base64WSEncode(self.primary_key.Header()
-                             + self.__InternalSign(data))
+    signature = self.primary_key.Header() + self.__InternalSign(data)
+    return encoder(signature) if encoder else signature
 
-  def AttachedSign(self, data, nonce):
+  def AttachedSign(self, data, nonce, encoder=util.Base64WSEncode):
     """
     Sign given data and nonce and return a blob containing both data and
     signature
 
-    For message M, and nonce N, outputs Header|len(M)|M|Sig(Header|M|N).
+    For message M, and nonce N, outputs Header|len(M)|M|Sig(M|len(N)|N|VersionByte).
 
     @param data: message to be signed
     @type data: string
@@ -618,9 +617,10 @@ class Signer(Verifier):
     @return: signature on the data encoded as a Base64 string
     @rtype: string
     """
-    return util.Base64WSEncode(self.primary_key.Header()
-                       + util.PackByteArray(data)
-                       + self.__InternalSign(data, nonce))
+    signature = self.primary_key.Header() \
+                + util.PackByteArray(data) \
+                + self.__InternalSign(data, nonce)
+    return encoder(signature) if encoder else signature
 
 
   def __InternalSign(self, data, nonce = None):
@@ -653,7 +653,7 @@ class UnversionedSigner(UnversionedVerifier):
     """Only valid if purpose includes signing."""
     return purpose == keyinfo.SIGN_AND_VERIFY
 
-  def Sign(self, data):
+  def Sign(self, data, encoder=util.Base64WSEncode):
     """
     Sign given data and return corresponding signature. This signature
     contains no header or version information.
@@ -669,7 +669,8 @@ class UnversionedSigner(UnversionedVerifier):
     signing_key = self.primary_key
     if signing_key is None:
       raise errors.NoPrimaryKeyError()
-    return util.Base64WSEncode(signing_key.Sign(data))
+    signature = signing_key.Sign(data)
+    return encoder(signature) if encoder else signature
 
 
 SESSION_NONCE_SIZE = 16
@@ -728,12 +729,14 @@ class _Session(object):
     return session
 
   @property
-  @util.Memoize
   def crypter(self):
     """
     Returns a Crypter which can be used to encrypt and decrypt data using the session key.
     """
-    return Crypter(readers.StaticKeyReader(self.__session_key, keyinfo.DECRYPT_AND_ENCRYPT))
+    if not hasattr(self, '_crypter'):
+        self._crypter = Crypter(readers.StaticKeyReader(
+            self.__session_key, keyinfo.DECRYPT_AND_ENCRYPT))
+    return self._crypter
 
   @property
   def nonce(self):
@@ -775,11 +778,11 @@ class SessionEncrypter(object):
     """
     return self._encrypted_session_material
 
-  def Encrypt(self, plaintext):
+  def Encrypt(self, plaintext, encoder=util.Base64WSEncode):
     """
     Encrypts the given plaintext with the session key and returns the base 64-encoded result.
     """
-    return self._session.crypter.Encrypt(plaintext)
+    return self._session.crypter.Encrypt(plaintext, encoder)
 
 
 class SessionDecrypter(object):
@@ -791,12 +794,12 @@ class SessionDecrypter(object):
   def __init__(self, crypter, session_material):
     self._session = _Session.LoadPackedKey(crypter.Decrypt(session_material))
 
-  def Decrypt(self, ciphertext):
+  def Decrypt(self, ciphertext, decoder=util.Base64WSDecode):
     """
     Decrypts the given base 64-encoded ciphertext with the session key and returns the
     decrypted plaintext.
     """
-    return self._session.crypter.Decrypt(ciphertext)
+    return self._session.crypter.Decrypt(ciphertext, decoder)
 
 
 class SignedSessionEncrypter(object):
