@@ -1,4 +1,3 @@
-#!/usr/bin/python2.4
 #
 # Copyright 2008 Google Inc.
 #
@@ -48,10 +47,10 @@ except ImportError:
 # overideable crypt library selection
 ACTIVE_CRYPT_LIB = 'm2crypto' if EVP else 'pycrypto'
 
-import errors
-import keyczar
-import keyinfo
-import util
+from keyczar import errors
+from keyczar import keyczar
+from keyczar import keyinfo
+from keyczar import util
 
 #TODO: Note that simplejson deals in Unicode strings. So perhaps we should
 #modify all Read() methods to wrap data obtained from simplejson with str().
@@ -850,7 +849,8 @@ class RsaPrivateKey(PrivateKey):
     @rtype: string
     """
     emsa_encoded = util.MakeEmsaMessage(msg, self.size)
-    return util.BigIntToBytes(self.key.sign(emsa_encoded, None)[0])
+    byte_string = util.BigIntToBytes(self.key.sign(emsa_encoded, None)[0])
+    return util.PadBytes(byte_string, self.size/8 - len(byte_string))
 
   def Verify(self, msg, sig):
     """@see: L{RsaPublicKey.Verify}"""
@@ -1201,11 +1201,7 @@ class DecryptingStreamReader(object):
         self.__hmac_stream.Update(data_to_decrypt)
         self.__encrypted_buffer = self.__encrypted_buffer[len(data_to_decrypt):]
         decrypted_data = self.__cipher.decrypt(data_to_decrypt)
-        if not is_data_avail:
-          decrypted_data = self.__key._UnPad(decrypted_data)
-
-        self.__decrypted_buffer += decrypted_data
-
+        
         if not is_data_avail:
           if len(self.__encrypted_buffer) != util.HLEN:
             raise errors.ShortCiphertextError(len(self.__encrypted_buffer))
@@ -1215,7 +1211,10 @@ class DecryptingStreamReader(object):
           if not self.__key.hmac_key.VerifySignedData(current_sig_bytes, 
                                                       msg_sig_bytes):
             raise errors.InvalidSignatureError()
-
+          decrypted_data = self.__key._UnPad(decrypted_data)
+          
+        self.__decrypted_buffer += decrypted_data
+        
     if chars < 0:
       result = self.__decrypted_buffer
       self.__decrypted_buffer = ''
