@@ -20,7 +20,6 @@ Utility functions for keyczar package.
 """
 
 import base64
-import cPickle
 import codecs
 import functools
 import math
@@ -29,11 +28,7 @@ import struct
 import warnings
 import sys
 
-try:
-  # Import hashlib if Python >= 2.5
-  from hashlib import sha1
-except ImportError:
-  from sha import sha as sha1
+from hashlib import sha1
 
 try:
   # check for presence of blocking I/O module
@@ -53,7 +48,7 @@ from pyasn1.codec.der import decoder
 from pyasn1.codec.der import encoder
 from pyasn1.type import univ
 
-import errors
+from keyczar import errors
 
 try:
   from abc import ABCMeta, abstractmethod, abstractproperty
@@ -320,11 +315,17 @@ def RandBytes(n):
   # This function requires at least Python 2.4.
   return os.urandom(n)
 
+def ConverToBytes(value):
+  if isinstance(value, str):
+    value = value.encode("utf-8")
+
+  return value
+
 def Hash(*inputs):
   """Return a SHA-1 hash over a variable number of inputs."""
   md = sha1()
   for i in inputs:
-    md.update(i)
+    md.update(ConverToBytes(i))
   return md.digest()
 
 def PrefixHash(*inputs):
@@ -332,7 +333,7 @@ def PrefixHash(*inputs):
   md = sha1()
   for i in inputs:
     md.update(IntToBytes(len(i)))
-    md.update(i)
+    md.update(ConverToBytes(i))
   return md.digest()
 
 
@@ -354,7 +355,8 @@ def Base64WSEncode(s):
   @return: Base64 representation of s.
   @rtype: string
   """
-  return base64.urlsafe_b64encode(str(s)).replace("=", "")
+
+  return base64.urlsafe_b64encode(s).rstrip(b"=")
 
 
 def Decode(s):
@@ -796,7 +798,7 @@ class IncrementalBase64WSStreamReader(codecs.StreamReader, object):
         data = self.bytebuffer + newdata
         try:
           newchars, decodedbytes = self.decode(data, self.errors)
-        except UnicodeDecodeError, exc:
+        except UnicodeDecodeError as exc:
           if firstline:
             newchars, decodedbytes = self.decode(data[:exc.start], self.errors)
             lines = newchars.splitlines(True)
