@@ -1,4 +1,3 @@
-#!/usr/bin/python2.4
 #
 # Copyright 2011 LightKeeper LLC.
 #
@@ -21,8 +20,8 @@ A Writer supports writing metadata and key info for key sets.
 """
 import os
 
-import errors
-import util
+from keyczar import errors
+from keyczar import util
 
 def CreateWriter(location):
   """Factory function for Writers
